@@ -2,16 +2,14 @@
 # HLfit(Reaction ~ 1 + (Days|Subject), data = sleepstudy,HLmethod="ML")
 # HLfit(Reaction ~ Days + (Days|Subject), data = sleepstudy,HLmethod="ML")
 
-.calc_latentL <- function(compactcovmat, triangularL=TRUE, spprecBool, try_chol=FALSE, condnum=condnum) {
+.calc_latentL <- function(compactcovmat, use_tri_Nspprec=TRUE, spprecBool, try_chol=FALSE, condnum=condnum) {
   # returns a *t*crossfactor 
   ## L and L_Q are distinct matrices used jointly in a fit (L in ZAL...) andmust be deduced from each other.
   #  In sparse precision code We want chol_Q to be lower triangular (dtCMatrix can be Up or Lo) 
   #    to obtain a dtCMatrix by bdiag(list of lower tri dtCMatrices).
   # Beyond this, design_u may be lower or upper tri
-  #compactcovmat <- .CovRegulCor(compactcovmat) ## not quite elegant, but robust. (but now, by default 'compactcovmat' is not modified) 
   if (try_chol) {
     if (is.null(chol_crossfac <- attr(compactcovmat, "chol_crossfac"))) {
-      #compactcovmat <- .regularize_Wattr(compactcovmat, condnum=condnum) # would introduce a painful trade-off with "sph" on condnum.
       chol_crossfac <- try(chol(compactcovmat), silent=TRUE)
     }
     if ( ! inherits(chol_crossfac, "try-error")) return(list(design_u=t(chol_crossfac), 
@@ -19,11 +17,15 @@
                                                              compactcovmat=compactcovmat))
   }
   # ELSE:
-  compactcovmat <- .regularize_Wattr(compactcovmat, condnum=condnum) 
+  #compactcovmat <- .regularize_Wattr(compactcovmat, condnum=condnum) 
+  compactcovmat <- .smooth_regul(compactcovmat, epsi=.spaMM.data$options$tol_ranCoefs_inner["regul"]) 
+  #   value of epsi utlmately also affects whether bobyqa has to be run 
+  # .smooth_regul(compactcovmat, epsi=1/condnum) # appears to work except wild result for test sph transfo
+  # # _F I X M E_ if this new code stands, then we might be able to get rid of .regularize_Wattr()
   d_regul <- attr(compactcovmat,"d_regul")
   esys <- attr(compactcovmat,"esys")
-  if (spprecBool || triangularL) { ## Competing for clumsy code prize... ## sparse version elsewhere
-    ## need triangular factor for spprec case in particular hence the default value of triangularL
+  if (spprecBool || use_tri_Nspprec) { ## Competing for clumsy code prize... ## sparse version elsewhere
+    ## need triangular factor for spprec case in particular hence the default value of use_tri_Nspprec
     crossfac_prec <- .Dvec_times_matrix(sqrt(1/d_regul), t(esys$vectors))
     qrblob <- qr(crossfac_prec)
     crossfac_prec <- qr.R(qrblob) # applying .lmwithQR() systematically (badly) affects numerical precision
@@ -72,9 +74,33 @@
   ## This is not provided by this function
 }
 
+.makelong_bigq <- function(Lcompact, longsize) { ## ad hoc version with restricted usage
+  n_levels <- longsize/ncol(Lcompact)
+  longLv <- gmp::as.bigq(diag(nrow=longsize))
+  pos_template <- matrix(FALSE,nrow=longsize,ncol=longsize)
+  for (it in seq_len(ncol(Lcompact))) {
+    urange1 <- (it-1)*n_levels + seq(n_levels)
+    positions <- pos_template
+    positions[cbind(urange1,urange1)] <- TRUE
+    longLv[[]][which(as.vector(positions))] <- Lcompact[it,it]
+    for (jt in seq_len(it-1)) {
+      urange2 <- (jt-1)*n_levels + seq(n_levels)
+      positions <- pos_template
+      positions[cbind(urange1,urange2)] <- TRUE
+      longLv[[]][which(as.vector(positions))] <- Lcompact[it,jt]
+      positions <- pos_template
+      positions[cbind(urange2,urange1)] <- TRUE
+      longLv[[]][which(as.vector(positions))] <- Lcompact[jt,it]
+    }
+  }
+  return(longLv) 
+} ## end def .makelong_bigq
+
 ## Build Lmatrix between all pairs of u_h (nr*nr) from parameter estimates (2*2) for the design matrix, longsize is final dim of matrix
 .makelong <- function(Lcompact, ## may also be a compact precmat, symmetric rather than triangular
                       longsize,as_matrix=FALSE,template=NULL) { ## always returns a Matrix unless explicitly as_matrix
+  
+  if (inherits(Lcompact, "bigq")) return(.makelong_bigq(Lcompact, longsize))
   if ( ! is.null(template)) { ## allows efficient filling of long template
     template@x <- Lcompact[template@x]
     if (as_matrix) {
@@ -88,20 +114,16 @@
     return(template) 
   } else {
     n_levels <- longsize/ncol(Lcompact)
-    if (.spaMM.data$options$Zcolsbyrows) {
-      longLv <- do.call(Matrix::bdiag,rep(list(Lcompact),n_levels)) # more diagonal... no benefits (yet)
-    } else {
-      if (longsize>180L) {
-        longLv <- Diagonal(n=longsize) ## declaration ## FIXME with some effort, we could produce better sparse code.
-      } else longLv <- diag(nrow=longsize) ## declaration
-      for (it in seq_len(ncol(Lcompact))) {
-        urange1 <- (it-1)*n_levels + seq(n_levels)
-        longLv[cbind(urange1,urange1)] <- Lcompact[it,it]
-        for (jt in seq_len(it-1)) {
-          urange2 <- (jt-1)*n_levels + seq(n_levels)
-          longLv[cbind(urange1,urange2)] <- Lcompact[it,jt]
-          longLv[cbind(urange2,urange1)] <- Lcompact[jt,it]
-        }
+    if (longsize>180L) {
+      longLv <- Diagonal(n=longsize) ## declaration ## FIXME with some effort, we could produce better sparse code.
+    } else longLv <- diag(nrow=longsize) ## declaration
+    for (it in seq_len(ncol(Lcompact))) {
+      urange1 <- (it-1)*n_levels + seq(n_levels)
+      longLv[cbind(urange1,urange1)] <- Lcompact[it,it]
+      for (jt in seq_len(it-1)) {
+        urange2 <- (jt-1)*n_levels + seq(n_levels)
+        longLv[cbind(urange1,urange2)] <- Lcompact[it,jt]
+        longLv[cbind(urange2,urange1)] <- Lcompact[jt,it]
       }
     }
     if (as_matrix) {
@@ -117,32 +139,42 @@
     }
     return(longLv) 
   }
-  return(longLv) 
+  #return(longLv)
 } ## end def makelong
 
 .makeCovEst1 <- function(u_h,ZAlist,cum_n_u_h,prev_LMatrices,
-                         var_ranCoefs,w.resid,processed,phi_est,
+                         var_ranCoefs,
+                         w.resid, # not variable within the function
+                         processed,phi_est,
                          #family, ## ignored
                         as_matrix,v_h, MakeCovEst_pars_not_ZAL_or_lambda,
-                        init_ranCoefs
+                        init_ranCoefs,
+                        prev_lambda_est
 ) {
+  H_global_scale <- .calc_H_global_scale(w.resid)
+  weight_X <- .calc_weight_X(w.resid, H_global_scale) ## sqrt(s^2 W.resid) ## should not affect the result up to precision
   rC_transf_inner <- .spaMM.data$options$rC_transf_inner
   nrand <- length(ZAlist)
   locX.Re <- processed$X.Re ## may be NULL 
   if (is.null(locX.Re)) locX.Re <- processed$AUGI0_ZX$X.pv
-  if ( ncol(locX.Re) ) { lik_obj="p_bv"} else lik_obj="p_v" 
+  if ( ncol(locX.Re) ) { lik_obj <- "p_bv"} else lik_obj <- "p_v" 
   updated_LMatrices <- working_LMatrices <- prev_LMatrices
   Xi_cols <- attr(ZAlist,"Xi_cols")
-  loc_lambda_est <- numeric(length(u_h))
+  loc_lambda_est <- prev_lambda_est # estimates not provided by the loop are necessary when (! augZXy_cond) -> .solve_IRLS_as_ZX()
   spprecBool <- processed$sparsePrecisionBOOL
-  use_tri <- .spaMM.data$options$use_tri_for_makeCovEst
-  if (spprecBool || use_tri) {
-    condnum <- .spaMM.data$options[["condnum_for_latentL_inner"]]
-  } else condnum <- .spaMM.data$options[["condnum_for_latentL"]]
-  try_chol <- ( ! spprecBool ) && (.spaMM.data$options[["rC_transf_inner"]]=="chol") ## "sph" could be combined with chol latentL but this does not appear to be optimal.
+  use_tri_Nspprec <- .spaMM.data$options$use_tri_for_makeCovEst
+  verbose <- processed$verbose["TRACE"]
+  if (spprecBool) {
+    condnum <- .spaMM.data$options[["condnum_for_latentL_spprec"]] # may be higher here than in  .process_ranCoefs()
+    #cat(condnum)
+  } else if (use_tri_Nspprec) {
+    condnum <- .spaMM.data$options[["condnum_for_latentL_tri_inner"]]
+  } else condnum <- .spaMM.data$options[["condnum_for_latentL_other"]]
+  try_chol <- ( ! spprecBool ) # && (.spaMM.data$options[["rC_transf_inner"]]=="chol") ## "sph" works with chol latentL now
   augZXy_cond <- attr(processed$augZXy_cond,"inner")
   test <- FALSE
   #test <- TRUE
+  pars_for_conv_corr <- vector("list",nrand)
   for (rt in seq_len(nrand)) {
     if ( var_ranCoefs[rt]) { ## inner estimation of cov mat of u_h 
       Xi_ncol <- Xi_cols[rt]
@@ -151,9 +183,9 @@
       ##prevL <- attr(prev_LMatrices[[rt]],"latentL_blob")$design_u
       u.range <- (cum_n_u_h[rt]+1L):(cum_n_u_h[rt+1L])
       ########## brute force optimization
-      objfn <- function(trRancoef) {
+      objfn <- function(trRancoef) { ## to be MINimized
         compactcovmat <- .calc_cov_from_trRancoef(trRancoef, Xi_ncol, rC_transf=rC_transf_inner)
-        latentL_blob <- .calc_latentL(compactcovmat, triangularL=use_tri, spprecBool=spprecBool, try_chol= try_chol , condnum=condnum)
+        latentL_blob <- .calc_latentL(compactcovmat, use_tri_Nspprec=use_tri_Nspprec, spprecBool=spprecBool, try_chol= try_chol , condnum=condnum)
         # Build Xscal
         working_LMatrices[[rt]] <- .makelong(latentL_blob$design_u,longsize=ncol(ZAlist[[rt]]), 
                                              template=processed$ranCoefs_blob$longLv_templates[[rt]]) ## the variances are taken out in $d
@@ -161,14 +193,15 @@
         locZAL <- .compute_ZAL(XMatrix=working_LMatrices, ZAlist=ZAlist, as_matrix=as_matrix) 
         # w.ranef argument
         loc_lambda_est[u.range] <- .make_long_lambda(latentL_blob$d, n_levels, Xi_ncol) 
-        if (.spaMM.data$options$rC_transf_inner!="chol") loc_lambda_est[loc_lambda_est<1e-08] <- 1e-08
+        ## latentL_blob$d may be a rep(1,...) in all calls to this fn. In which cas, below,
+        ## locwranefblob and ZAL_scaling are constant; non-constant code begisn with Xscal(locZAL...)
+        #if (.spaMM.data$options$rC_transf_inner!="chol") loc_lambda_est[loc_lambda_est<1e-08] <- 1e-08 # (two occurrences)
         if (augZXy_cond || test) {
           ####################################################################################################
           n_u_h <- length(u_h)
           # we don't want anything specific on u_h values:
           locwranefblob <- .updateW_ranefS(processed$cum_n_u_h, processed$rand.families, lambda=loc_lambda_est, 
                                            u_h=rep(NA,n_u_h),v_h=rep(NA,n_u_h)) ## indeed
-          H_global_scale <- .calc_H_global_scale(w.resid)
           ZAL_scaling <- 1/sqrt(locwranefblob$w.ranef*H_global_scale) ## Q^{-1/2}/s
           Xscal <- .make_Xscal(ZAL=locZAL, ZAL_scaling = ZAL_scaling, AUGI0_ZX=processed$AUGI0_ZX)
           if (inherits(Xscal,"Matrix")) { # same type as ZAL
@@ -176,7 +209,6 @@
           } else {
             mMatrix_method <- .spaMM.data$options$matrix_method
           }
-          weight_X <- .calc_weight_X(w.resid, H_global_scale) ## sqrt(s^2 W.resid) ## should not affect the result up to precision
           sXaug <- do.call(mMatrix_method,
                            list(Xaug=Xscal, weight_X=weight_X, w.ranef=locwranefblob$w.ranef, H_global_scale=H_global_scale))
           ####################################################################################################
@@ -185,19 +217,19 @@
           ####################################################################################################
           locw.ranefSblob <- .updateW_ranefS(cum_n_u_h,processed$rand.families,lambda=loc_lambda_est,u_h,v_h) 
           locarglist <- c(MakeCovEst_pars_not_ZAL_or_lambda, list(ZAL=locZAL, lambda_est=loc_lambda_est, wranefblob=locw.ranefSblob))
-          # it would be really nonsens to compute the objective for constant u_h;
+          # it would be really nonsense to compute the objective for constant u_h;
           ## the u_h should be evaluated at the BLUPs (or equivalent) for given parameters
           auglinmodblob <- do.call(".solve_IRLS_as_ZX",locarglist)
           ## FR->FR but its not clear that non-standard REML is handled by calc_APHLs_from_ZX !!
           ####################################################################################################
-          obj_augZX <- .calc_APHLs_from_ZX(sXaug=NULL, # may be NULL
-                                           auglinmodblob=auglinmodblob,  # may be NULL
+          obj_augZX <- .calc_APHLs_from_ZX(sXaug=NULL,
+                                           auglinmodblob=auglinmodblob, # including $lambda_est, needed here
                                            phi_est=phi_est,
                                            processed=processed,
                                            which=lik_obj)[[lik_obj]]
         } 
         if ( augZXy_cond || test) {
-          obj_augZXy <- .calc_APHLs_by_augZXy_or_sXaug(sXaug=sXaug, 
+          obj_augZXy <- .calc_APHLs_by_augZXy_or_sXaug(sXaug=sXaug, # with this we don't need a lambda_est argument.
                                            auglinmodblob=NULL, 
                                            phi_est=phi_est,
                                            processed=processed,
@@ -220,37 +252,25 @@
                                                rC_transf=rC_transf_inner)
       lowerb <- trRancoef_LowUp$lower
       upperb <- trRancoef_LowUp$upper
-      if (TRUE) {
-        nloptr_controls <- spaMM.getOption("nloptr") 
-        if (is.null(nloptr_controls$maxeval)) nloptr_controls$maxeval <-  eval(.spaMM.data$options$maxeval,list(initvec=init_trRancoef))
-        if (is.null(nloptr_controls$xtol_abs)) {
-          nloptr_controls$xtol_abs <- eval(.spaMM.data$options$xtol_abs, 
-                                           list(LowUp=trRancoef_LowUp, rC_transf=rC_transf_inner) ) 
-        }
-        optr <- nloptr::nloptr(x0=init_trRancoef,eval_f=objfn,lb=lowerb,ub=upperb,
-                       opts=nloptr_controls)
-        #browser()
-        while (optr$status==5L) { ## 5 => termination bc maxeval has been reached
-          prevlik <- optr$objective
-          reinit <- pmax(lowerb,pmin(upperb,optr$solution))
-          optr <- nloptr::nloptr(x0=reinit,eval_f=objfn,lb=lowerb,ub=upperb,
-                                 opts=nloptr_controls)
-          loc_ftol <- max(1e-8, optr$options$ftol_abs)
-          if (- optr$objective < - prevlik+loc_ftol) break ## no progress in <= maxeval iterations
-        }
-      } else { ## deprecated:
-        ################# OPTIM
-        parscale <- (upperb-lowerb)
-        optr <- optim(init_trRancoef,objfn,lower=lowerb,upper=upperb,method="L-BFGS-B", ## deprecated
-                      control=list(parscale=parscale))
+      Optimizer <- .spaMM.data$options$optim_inner
+      if (Optimizer==".safe_opt") { # not that optimizer is not preprocessed (and that would require some thoughts)
+        optr <- .safe_opt(init=init_trRancoef, objfn=objfn, lower=lowerb, upper=upperb, verbose=verbose,
+                          adjust_init=trRancoef_LowUp$adjust_init) 
+      } else if (Optimizer=="nloptr") {
+        optr <- .optim_by_nloptr(initvec=init_trRancoef,lowerb=lowerb,upperb=upperb,objfn_locoptim=objfn,
+                                 local_control=NULL,LowUp=trRancoef_LowUp)
+      } else if (Optimizer=="bobyqa") {
+        optr <- .optim_by_bobyqa(initvec=init_trRancoef,lowerb=lowerb,upperb=upperb,objfn_locoptim=objfn,
+                                 local_control=NULL)
+        optr$objective <- optr$fval
         optr$solution <- optr$par
       }
       ################# 
       ## reproduces representation in objfn
       compactcovmat <- .calc_cov_from_trRancoef(optr$solution, Xi_ncol, rC_transf=rC_transf_inner)
-      latentL_blob <- .calc_latentL(compactcovmat, triangularL=use_tri, spprecBool=spprecBool, try_chol= try_chol , condnum=condnum)
+      latentL_blob <- .calc_latentL(compactcovmat, use_tri_Nspprec=use_tri_Nspprec, spprecBool=spprecBool, try_chol= try_chol , condnum=condnum)
       loc_lambda_est[u.range] <- .make_long_lambda(latentL_blob$d, n_levels, Xi_ncol) 
-      if (.spaMM.data$options$rC_transf_inner!="chol") loc_lambda_est[loc_lambda_est<1e-08] <- 1e-08 
+      #if (.spaMM.data$options$rC_transf_inner!="chol") loc_lambda_est[loc_lambda_est<1e-08] <- 1e-08 
       next_LMatrix <- .makelong(latentL_blob$design_u,longsize=ncol(ZAlist[[rt]]), 
                                 template=processed$ranCoefs_blob$longLv_templates[[rt]]) ## il faut updater pour estimer les ranef correctement...
       attr(next_LMatrix,"latentL_blob") <- latentL_blob ## kept for updating in next iteration and for output
@@ -259,10 +279,15 @@
                                                 ## type is "(.|.)" if LMatrix is for random slope ## ajout 2015/06
                                                 type= attr(ZAlist,"exp_ranef_types")[rt] )
       attr(next_LMatrix, "corr.model") <- "random-coef"
-      updated_LMatrices[[rt]] <- next_LMatrix # (fixme ??) working_LMatrices[[rt]] <- 
+      updated_LMatrices[[rt]] <- next_LMatrix # (fixme ??) working_LMatrices[[rt]] <-
+      ### in the HLfit6 example, one element of compactcovmat appears to move without affecting logL, preventing compactcovmat convergence
+      ## the optpars for "chol" appear to converge, but not those for "sph"
+      #pars_for_conv_corr[[rt]] <- compactcovmat[lower.tri(compactcovmat,diag=TRUE)] 
+      pars_for_conv_corr[[rt]] <- optr$solution 
     } else updated_LMatrices[rt] <- list(NULL) ## this erases Matern, AR1, and other fixed LMatrices, so updated_LMatrices is not to be used in objfn()   
   } ## loop on rt = ranefs
+  if (verbose["TRACE"]>1L) print(optr)
   return(list(updated_LMatrices=updated_LMatrices, 
               next_lambda_est=loc_lambda_est,
-              optr_par=optr$solution))
+              info_for_conv_rC=list(obj=optr$objective, ranCoefs=unlist(pars_for_conv_corr))))
 } ## end def makeCovEst1
