@@ -105,7 +105,7 @@ corrHLfit_body <- function(processed, ## possibly a list of environments
       optPars <- .new_locoptim(init.optim, LowUp=LowUp,anyHLCor_obj_args=anyHLCor_obj_args,
                                objfn_locoptim=.objfn_locoptim,
                                control=control.corrHLfit, 
-                               user_init_optim=user_init_optim)
+                               user_init_optim=user_init_optim, verbose=verbose[["TRACE"]])
     }
   }
   if (!is.null(optPars)) {
@@ -120,7 +120,7 @@ corrHLfit_body <- function(processed, ## possibly a list of environments
   }
   ranPars_in_refit <- .expand_hyper(ranPars_in_refit, processed$hyper_info, moreargs=moreargs)
   HLCor.args$ranPars <- ranPars_in_refit
-  # if processed is an envir, the following is not local to anyHLCor_obj_args$processed but change processed globally
+  # not local to anyHLCor_obj_args$processed: change processed globally
   .assignWrapper(HLCor.args$processed,"return_only <- NULL") 
   .assignWrapper(HLCor.args$processed,"verbose['warn'] <- TRUE") ## important!
   hlcor <- do.call("HLCor",HLCor.args) ## recomputation post optimization (or only computation, if length(lower)=0)
