@@ -57,7 +57,7 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
   }
   # ELSE
   if ( is.null(BLOB$R_R_v) && which %in% c("absdiag_R_v","d2hdv2","solve_d2hdv2","hatval_Z","R_scaled_v_h_blob",
-                                           "logdet_R_scaled_v")) {
+                                           "logdet_R_scaled_v","Mg_invH_g")) {
     seq_n_u_h <- seq_len(attr(sXaug,"n_u_h"))
     # remove $u_h_cols_on_left code in version 2.1.61 since there is a bug in it: further code may require 
     #  no nontrivial perm_R_v, so we remove perm_R_v it from further code (as in Matrix_QRP_CHM_scaled version)
@@ -83,16 +83,18 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
       } else BLOB$t_Q_scaled <- backsolve(BLOB$R_scaled,t(sXaug),transpose=TRUE)
     }
     return(colSums(BLOB$t_Q_scaled^2))
-  } else if (which=="hatval_Z") { ## Pdiag
+  } else if (which=="hatval_Z") { ## Pdiag; note that these leverages are constant wrt to any diagonal rescaling of the Z block
     if (is.null(BLOB$hatval_Z_)) {
       ## t(sXaug) is scaled such that the left block is an identity matrix, so we can work 
       ## on two separate blocks if the Cholesky is not permuted. Then 
-      lev_lambda <- backsolve(BLOB$R_R_v,diag(ncol(BLOB$R_R_v)),transpose=TRUE)
+      if (is.null(lev_lambda <- BLOB$inv_factor_wd2hdv2w)) {
+        lev_lambda <- BLOB$inv_factor_wd2hdv2w <- backsolve(BLOB$R_R_v,diag(ncol(BLOB$R_R_v)),transpose=TRUE)
+      } 
       lev_lambda <- lev_lambda^2
       lev_lambda <- colSums(lev_lambda)
       n_u_h <- attr(sXaug,"n_u_h")
       phipos <- (n_u_h+1):nrow(sXaug)
-      lev_phi <- backsolve(BLOB$R_R_v, t(sXaug[phipos, seq_len(n_u_h) ]),transpose=TRUE) ## fixme the t() may still be costly
+      lev_phi <- .tcrossprod(BLOB$inv_factor_wd2hdv2w, sXaug[phipos, seq_len(n_u_h) ]) # backsolve(BLOB$R_R_v, t(sXaug[phipos, seq_len(n_u_h) ]),transpose=TRUE) ## fixme the t() may still be costly
       lev_phi <- lev_phi^2
       lev_phi <- colSums(lev_phi)
       BLOB$hatval_Z_ <-  list(lev_lambda=lev_lambda,lev_phi=lev_phi)
@@ -114,15 +116,27 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
       BLOB$R_scaled_v_h_blob <- list(R_scaled_v_h=BLOB$R_R_v, diag_pRtRp_scaled_v_h=diag_pRtRp_scaled_v_h)
     }
     return(BLOB$R_scaled_v_h_blob)
+  } else if (which=="R_beta_blob") {
+    if (is.null(BLOB$R_beta_blob)) {
+      n_u_h <- attr(sXaug,"n_u_h")
+      seq_n_u_h <- seq(n_u_h)
+      X <- as.matrix(sXaug[-seq_n_u_h,-seq_n_u_h]) ## follwoing code assuming it is dense...
+      R_beta <- .lmwithQR(X,yy=NULL,returntQ=FALSE,returnR=TRUE)$R_scaled
+      diag_pRtRp_beta <-  colSums(R_beta^2)
+      BLOB$R_beta_blob <- list(R_beta=R_beta,diag_pRtRp_beta=diag_pRtRp_beta)
+    }
+    return(BLOB$R_beta_blob)
   } else if (which=="d2hdv2") {
     stop("d2hdv2 requested")
     # don't forget that the factored matrix is not the augmented design matrix ! hence w.ranef needed here
     w.ranef <- attr(sXaug,"w.ranef")
     w_R_R_v <- .Matrix_times_Dvec(BLOB$R_R_v, sqrt(w.ranef))  #BLOB$R_R_v %*% diag(sqrt(w.ranef))
     BLOB$d2hdv2 <- - .crossprodCpp(w_R_R_v,yy=NULL)
-  } else if (which=="Mg_invH_g") {
+  } 
+  if (which=="Mg_invH_g") {
     return(sum(solve(BLOB$R_R_v,B)^2))
-  } else if (which=="Mg_solve_g") {
+  } 
+  if (which=="Mg_solve_g") {
     if (is.null(BLOB$invsqrtwranef)) BLOB$invsqrtwranef <- 1/sqrt(attr(sXaug,"w.ranef")) 
     seq_n_u_h <- seq_len(attr(sXaug,"n_u_h"))
     rhs <- B
@@ -131,11 +145,19 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
       rhs <- forwardsolve(BLOB$R_scaled, rhs, upper.tri = TRUE, transpose = TRUE)
     } else rhs <- forwardsolve(BLOB$R_scaled, rhs[BLOB$perm], upper.tri = TRUE, transpose = TRUE)
     return(sum(rhs^2))
-  } else if (which=="solve_d2hdv2") { ## R'R[seq_n_u_h, seq_n_u_h] gives -d2hd_scaled_v2.
+  } 
+  if (which=="Mg_invXtWX_g") { ## 
+    if (is.null(BLOB$XtWX)) BLOB$XtWX <- sXaug[-seq_n_u_h,-seq_n_u_h]
+    Mg_invXtWX_g <- crossprod(B,solve(BLOB$XtWX,B))[1L] # [1L] drops possible Matrix class...
+    return(Mg_invXtWX_g)
+  } 
+  if (which=="solve_d2hdv2") { ## R'R[seq_n_u_h, seq_n_u_h] gives -d2hd_scaled_v2.
     if ( is.null(B) ) {
       if ( is.null(BLOB$inv_d2hdv2)) {
         if (is.null(BLOB$invsqrtwranef)) BLOB$invsqrtwranef <- 1/sqrt(attr(sXaug,"w.ranef")) 
-        inv_d2hdv2 <- chol2inv(BLOB$R_R_v)
+        if (is.null(BLOB$inv_factor_wd2hdv2w)) {
+          inv_d2hdv2 <- chol2inv(BLOB$R_R_v)
+        } else inv_d2hdv2 <- .crossprod(BLOB$inv_factor_wd2hdv2w)
         inv_d2hdv2 <- .m_Matrix_times_Dvec(inv_d2hdv2, BLOB$invsqrtwranef) ## _m_atrix sweep 2L
         BLOB$inv_d2hdv2 <- - .Dvec_times_matrix(BLOB$invsqrtwranef,inv_d2hdv2)
       }
@@ -146,7 +168,9 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
         if (is.matrix(B)) {
           rhs <- .Dvec_times_matrix(BLOB$invsqrtwranef,B)
         } else rhs <- BLOB$invsqrtwranef * B
-        rhs <- backsolve(BLOB$R_R_v, forwardsolve(BLOB$R_R_v, rhs, upper.tri = TRUE, transpose = TRUE))
+        if (is.null(BLOB$inv_factor_wd2hdv2w)) {
+          rhs <- backsolve(BLOB$R_R_v, forwardsolve(BLOB$R_R_v, rhs, upper.tri = TRUE, transpose = TRUE))
+        } else rhs <- .crossprod(BLOB$inv_factor_wd2hdv2w) %*% rhs
         if (is.matrix(rhs)) {
           rhs <- .Dvec_times_matrix(BLOB$invsqrtwranef,rhs)
         } else rhs <- BLOB$invsqrtwranef * rhs
@@ -224,6 +248,13 @@ get_from_MME.sXaug_EigenDense_QRP_Chol_scaled <- function(sXaug,which="",szAug=N
                    list(dVscaled = .damping_to_solve(X=R_scaled_v_h_blob$R_scaled_v_h, dampDpD=dampDpD, rhs=LMrhs), 
                         dampDpD = dampDpD) 
                  },
+                 "LevMar_step_beta" = {
+                   if ( ! length(LMrhs)) stop("LevMar_step_beta called with 0-length LMrhs: pforpv=0?")
+                   R_beta_blob <- .sXaug_EigenDense_QRP_Chol_scaled(sXaug,which="R_beta_blob")
+                   dampDpD <- damping*R_beta_blob$diag_pRtRp_beta
+                   list(dbeta = .damping_to_solve(X=R_beta_blob$R_beta, dampDpD=dampDpD, rhs=LMrhs), 
+                        dampDpD = dampDpD) 
+                 },
                  ## all other cases:
                  .sXaug_EigenDense_QRP_Chol_scaled(sXaug,which=which,szAug=szAug,B=B)
   )
