@@ -4,7 +4,7 @@ stripHLfit <- function(object,...) {
   if (inherits(object,"HLfit")) {
     ######## clean matrices
     stripnames <- names(object$envir)
-    stripnames <- setdiff(stripnames,c("G_CHMfactor","chol_Q","ZtW","qrXa","X_scaling","ZAfix")) ## exception for those that are not reconstructed on request
+    stripnames <- setdiff(stripnames,c("G_CHMfactor","sXaug","chol_Q","ZtW","qrXa","X_scaling","ZAfix")) ## exception for those that are not reconstructed on request
     for (st in stripnames) object$envir[[st]] <- NULL
     if (object$spaMM.version <= "1.10.3") { ## for later version, this part of code documents some obscure issues.
       ######### family matters ...
